#ifndef _RaveInterface_BTagFactory_H_
#define _RaveInterface_BTagFactory_H_

#include <rave/RaveTrack.h>
#include <rave/RaveVertex.h>
#include <rave/RaveCalibration.h>
#include <rave/VertexFactory.h>

class RaveReconstructor;

namespace rave
{
  class BTagger;

/** 
 *  The class that is central to using the rave library.
 *  It builds the RaveVertices from a set of RaveTracks.
 *  \paramname method is a string that describes which
 *  algorithms should be employed and which parameter
 *  settings should be chosen.
 *  The default is chosen such that something
 *  "sensible" is done in most use cases.
 */

class BTagFactory 
{
  public:
    BTagFactory ( const RaveReconstructor & r );
    BTagFactory ( const VertexFactory & v );
    BTagFactory ( const BTagFactory & o );
    ~BTagFactory();

    double tag ( const std::vector < Track > & tracks ) const;
    double tag ( const std::vector < Track > & tracks, const Vertex & primary,
                 const Vector3D & jet ) const;
    void teach ( const std::vector < Track > & tracks, char tag );
    void teach ( const std::vector < Track > & tracks, const Vertex & primary,
                 const Vector3D & jet, char tag );

    bool hasBTagging() const; //< has btagging code?
    /// calibrate the btagger
    void calibrate ( const rave::Calibration & ) const;
    /// get the calibration, optionally "finish the teacher"
    /// (=normalize) beforehand.
    rave::Calibration getCalibration() const;

  private:
    RaveReconstructor * theRector;
    mutable rave::BTagger * theTagger;
};

}

#endif
