#ifndef _BasicRaveBasicTrack_H_
#define _BasicRaveBasicTrack_H_

#include <rave/RaveVector6D.h>
#include <rave/RaveCovariance6D.h>
#include <rave/RaveCharge.h>
#include <rave/ReferenceCounted.h>
#include <rave/CopyUsingNew.h>
#include <rave/ProxyBase.h>
#include <rave/AbstractRaveTrack.h>
#include <string>
#include <vector>

namespace rave
{
/** \class BasicRaveBasicTrack defines the input to the rave library.
 * Fixme, it's more something like a RaveTrajectoryState,
 * than a BasicRaveBasicTrack.
 */

class BasicTrack : public ReferenceCounted, public AbstractTrack
{
  public:
    typedef ProxyBase< BasicTrack, CopyUsingNew<BasicTrack> > Proxy;

    /**
     *  Create a BasicRaveBasicTrack from a 6-dimensional euclidean trajectory state
     *  plus covariance matrix, plus charge, plus an optional
     *  pointer to the "original track", whatever that means
     */
    BasicTrack( const Vector6D &, const Covariance6D &, Charge,
           double chi2, double ndof,
           void * originaltrack = 0, std::string tag="" );

    BasicTrack ( const std::vector < rave::BasicTrack > & );

    BasicTrack();

    Charge charge() const;
    const Vector6D & state() const;
    const Covariance6D & error() const;
    void * originalObject() const;
    std::string tag() const;
    int id() const;
    double chi2() const;
    double ndof() const;

    bool isValid() const;

    BasicTrack * clone () const;

    std::vector < rave::BasicTrack > components() const;

    bool operator< ( const rave::BasicTrack & ) const;

  private:
    Vector6D theState;
    Covariance6D theError;
    Charge theCharge;
    void * thePointer;
    std::string theTag;
    int theId;
    bool theIsValid;
    double theChi2;
    double theNdof;
    std::vector < BasicTrack > theComponents;
};

}

#endif
