#ifndef _BasicRaveVertex_H_
#define _BasicRaveVertex_H_

#include <rave/RaveTrack.h>
#include <rave/RavePoint3D.h>
#include <rave/RaveCovariance3D.h>
#include <rave/CopyUsingNew.h>
#include <rave/ProxyBase.h>
#include <rave/AbstractRaveVertex.h>
#include <utility>
#include <vector>

namespace rave 
{

class BasicVertex : public ReferenceCounted, public AbstractVertex
{
  /**
   *  The output of the rave library: a reconstructed vertex.
   *  A position in 3d space, plus its error.
   *  Along with the (refitted!!) tracks and
   *  the association probabilities of tracks belonging to this vertex.
   *  also a (fractional) number of degrees of freedom (ndf)
   *  and chi-squared.
   */
  public:  
    typedef ProxyBase< BasicVertex, CopyUsingNew<BasicVertex> > Proxy;
    const Point3D & position() const;
    const Covariance3D & error() const;
    const std::vector < std::pair < double, Track > > & weightedTracks() const;
    std::vector < Track > tracks() const;
    double ndf() const;
    double chiSquared() const;
    int id() const;
    bool isValid() const;
    std::vector < BasicVertex > components() const;

  public:
    BasicVertex(); // default constructor, produces invalid vertex.
    BasicVertex( const Point3D &, const Covariance3D &,
            const std::vector < std::pair < double, Track > > & tracks, double ndf, double chi2 );
    BasicVertex( const Point3D &, const Covariance3D &,
            const std::vector < Track > & tracks, double ndf, double chi2 );

  private:
    Point3D thePoint;
    Covariance3D theError;
    std::vector < std::pair < double, Track > > theTracks;
    std::vector < BasicVertex > theComponents;
    double theNdf;
    double theChi2;
    int theId;
    bool theValid;
};

}

#endif
