#ifndef _RaveTrack_H_
#define _RaveTrack_H_

#include <rave/RaveVector6D.h>
#include <rave/RaveCovariance6D.h>
#include <rave/RaveCharge.h>
#include <rave/BasicRaveTrack.h>
#include <string>

namespace rave
{

/** \class RaveTrack defines the input to the rave library.
 * Fixme, it's more something like a RaveTrajectoryState,
 * than a RaveTrack.
 */

class Track : private BasicTrack::Proxy
{
  typedef BasicTrack::Proxy Base;
  public:

    /**
     *  Create a RaveTrack from a 6-dimensional euclidean trajectory state
     *  plus covariance matrix, plus charge, plus an optional
     *  pointer to the "original track", whatever that means
     */
    Track( const Vector6D &, const Covariance6D &, Charge,
           double chi2, double ndof,
           void * originaltrack = 0, std::string tag="" );

    Track( const Vector6D &, const Covariance6D &, Charge,
           double chi2, double ndof,
           int originaltrack, std::string tag="" );

    Track ( const BasicTrack & );
    Track();

    Charge charge() const;
    const Vector6D & state() const;
    const Covariance6D & error() const;
    void * originalObject() const;
    std::string tag() const;
    int id() const;

    double chi2() const;
    double ndof() const;

    bool isValid() const;
    std::vector < Track > components() const;

    bool operator< ( const rave::Track & ) const;
    bool operator== ( const rave::Track & ) const;
};

}

#endif
