#ifndef _RaveVertex_H_
#define _RaveVertex_H_

#include <rave/RaveTrack.h>
#include <rave/RavePoint3D.h>
#include <rave/RaveCovariance3D.h>
#include <rave/BasicRaveVertex.h>
#include <utility>
#include <vector>

namespace rave 
{

class Vertex : private BasicVertex::Proxy
{
  typedef BasicVertex::Proxy Base;
  /**
   *  The output of the rave library: a reconstructed vertex.
   *  A position in 3d space, plus its error.
   *  Along with the (refitted!!) tracks and
   *  the association probabilities of tracks belonging to this vertex.
   *  also a (fractional) number of degrees of freedom (ndf)
   *  and chi-squared.
   */
  public:
    Vertex(); //< default constructor, creating invalid vertex. 
    Vertex( const BasicVertex & );
    const Point3D & position() const;
    const Covariance3D & error() const;
    const std::vector < std::pair < double, Track > > & weightedTracks() const;
    std::vector < Track > tracks() const;
    double ndf() const;
    double chiSquared() const;
    int id() const;
    bool isValid() const;
    bool operator< ( const rave::Vertex & ) const;
    bool operator== ( const rave::Vertex & ) const;

    std::vector < Vertex > components() const;

    Vertex( const Point3D &, const Covariance3D &,
            const std::vector < std::pair < double, Track > > & tracks, double ndf, double chi2 );
    Vertex( const Point3D &, const Covariance3D &,
            const std::vector < Track > & tracks, double ndf, double chi2 );
};

}

#endif
