#ifndef VertexReco_Vertex_H
#define VertexReco_Vertex_H
#include "DataFormats/Math/interface/Error.h" 
#include "DataFormats/Math/interface/Point3D.h" 
#include "DataFormats/TrackReco/interface/TrackFwd.h"

namespace reco {
  class Vertex {
    public:
      enum { dimension = 3 };
      typedef math::Error<dimension> CovarianceMatrix;
      typedef math::Error<dimension> Error;
      typedef math::XYZPoint Point;

      Vertex ( const Point &, const Error &, double chi2, double ndof, size_t size );
      Vertex ();
      virtual ~Vertex() {};

      double chi2() const;
      double ndof() const;
      double x() const;
      double y() const;
      double z() const;
      double normalizedChi2() const;
      
      size_t tracksSize() const;
      void add ( const reco::TrackRef &, double w=1.0 );
      track_iterator tracks_begin() const;
      track_iterator tracks_end() const;

      void removeTracks();

      const Point & position() const;
      CovarianceMatrix covariance() const;
    private:
      Point p_;
      CovarianceMatrix c_;
      double chi2_;
      double ndf_;
      int size_;
      std::vector < reco::TrackRef > tracks_;
  };
}

#endif
