#ifndef MessageLogger_MessageLogger_h
#define MessageLogger_MessageLogger_h

// Rave Version of MessageLogger

#include <memory>
#include <string>
#include <iostream>
#include "FWCore/MessageLogger/interface/Colors.h"

namespace edm  {
  typedef std::ostream ErrorObj;
  class LogBase
  {
    public:
      enum Level { Debug=0, Info, Warning, Error };
      LogBase ( Level level );
      ~LogBase();
      LogBase & operator<< ( std::ostream&(*f)(std::ostream&));
      LogBase & operator<< ( std::ios_base&(*f)(std::ios_base&) );
      template< class T > LogBase & operator<< (T const & t)
      { 
        std::cout << t;
        return *this;
      }
    protected:
      Level level_;
  };

  class LogWarning : public LogBase
  {
  public:
    explicit LogWarning( std::string const & id ) ;
  };

  class LogError : public LogBase
  {
  public:
    explicit LogError( std::string const & id ) ;
  };

  class LogInfo : public LogBase
  {
  public:
    explicit LogInfo( std::string const & id ) ;
  };

  /* Commented out, because it is ugly, together with a LogDebug macro ...
  class LogDebug : public LogBase
  {
  public:
    explicit LogDebug( std::string const & id ) ;
  };*/

  class LogDebug_ : public LogBase
  {
  public:
    explicit LogDebug_( std::string const & id, std::string const & file, int line ) ;
    explicit LogDebug_( std::string const & id );
  };
}

#define LogDebug(id) edm::LogDebug_ ( id , __FILE__ , __LINE__ )

#endif  // MessageLogger_MessageLogger_h

