#ifndef ParameterSet_ParameterSet_h
#define ParameterSet_ParameterSet_h


/* our own simplistic version of ParameterSet. Just enough to get through
 * with ConfigurableVertexReco ...
 */

#include "Environment/Configuration/interface/MultiType.h"
#include <string>
#include <vector>
#include <map>
#include <ostream>
#include <exception>


// ----------------------------------------------------------------------
// contents

namespace edm {

  class ParameterSet {
  public:
    void augment( ParameterSet const & from );
    std::vector<std::string> getParameterNames() const;

    template <class T> T getParameter ( const std::string & s ) const
    {
      if (!(theMap.count(s)))
      {
        throw std::exception ();
      }
      return (T) (theMap[s]);
    }

    template <class T> T getParameter( const std::string & s , const T & def ) const
    {
      if (!(theMap.count(s)))
      {
        return def;
      }
      return (T) (theMap[s]);
    }

    template <class T> void addParameter( const std::string & s, const T & def ) const
    {
      theMap[s]=def;
    }

    template <class T> T getUntrackedParameter( const std::string & s ) const
    {
      return getParameter<T> ( s );
    }
    template <class T> T getUntrackedParameter( const std::string & s , const T & def ) const
    {
      return getParameter<T> ( s, def );
    }

    template <class T> void addUntrackedParameter( const std::string & s, const T & def ) const
    {
      addParameter<T> ( s, def );
    }

    std::string toString() const;

    bool isEmpty() const;

    private:
      mutable std::map < std::string, dataharvester::MultiType > theMap;
      mutable std::map < std::string, edm::ParameterSet > theNested;
  }; // ParameterSet


  template<> inline ParameterSet ParameterSet::getParameter<edm::ParameterSet>
       (std::string const& name, edm::ParameterSet const& defaultValue) const
  {
    if (!theNested.count(name) ) return defaultValue;
    return theNested[name];
  }

  template<> inline ParameterSet ParameterSet::getUntrackedParameter<edm::ParameterSet>
       (std::string const& name, edm::ParameterSet const& defaultValue) const
  {
    return getParameter<edm::ParameterSet>(name,defaultValue);
  }

  template<> inline ParameterSet ParameterSet::getParameter<edm::ParameterSet>
     (std::string const& name) const
  {
    if (!theNested.count(name) ) throw std::exception();
    return theNested[name];
  }

  template<> inline ParameterSet ParameterSet::getUntrackedParameter<edm::ParameterSet>
     (std::string const& name) const
  {
    return getParameter<edm::ParameterSet>(name);
  }


  template<> inline void ParameterSet::addParameter<edm::ParameterSet>
     (std::string const& name, const edm::ParameterSet & s ) const
  {
    theNested[name]=s;
  }

  template<> inline void ParameterSet::addUntrackedParameter<edm::ParameterSet>
     (std::string const& name, const edm::ParameterSet & s ) const
  {
    addParameter<edm::ParameterSet>(name,s);
  }
}  // namespace edm

std::ostream & operator<< (std::ostream& os, const edm::ParameterSet & );

#endif
