#ifndef _RaveBTagger_H_
#define _RaveBTagger_H_

#include <string>
#include "RecoVertex/VertexPrimitives/interface/TransientVertex.h"
#include "TrackingTools/TransientTrack/interface/TransientTrack.h"
#include "DataFormats/Candidate/interface/Particle.h"
#include <rave/RaveCalibration.h>
#include "RaveBase/RaveEngine/interface/RaveReconstructor.h"
#include <vector>

/**
 *  The RaveBTagger - btagging in rave.
 *  The Rave BTagger separates the rave interface from
 *  the btagging implementation.
 *  As such, it is also responsible for producing
 *  "dummy output", if btagging is not compiled in.
 */

#ifdef WITH_BTAGGING
namespace combsv {
  class CombinedSVAlgorithm;
}

class NCategoriesTeacher;
#endif

namespace rave {
  class BTagger 
  {
    public:
      BTagger( const rave::Calibration & c, const RaveReconstructor * );
      BTagger ( const BTagger & );
      ~BTagger();
      double tag ( const std::vector < reco::TransientTrack > &,
                   const TransientVertex &,
                   const reco::Particle & ) const;
      double tag ( const std::vector < reco::TransientTrack > & ) const;
      void teach ( const std::vector < reco::TransientTrack > &,
                   const TransientVertex &,
                   const reco::Particle &, char tag ) const;
      void teach ( const std::vector < reco::TransientTrack > &, char tag ) const;

      rave::Calibration getCalibration() const;

      static bool hasBTagging();

      BTagger * clone() const;
    private:
      #ifdef WITH_BTAGGING
      combsv::CombinedSVAlgorithm * theAlgo;
      NCategoriesTeacher * theTeacher;
      std::vector < rave::CalibrationRun > theMetaInfo;
      #endif 
  };
}

#endif
