#ifndef XML_ALGORITHM_CALIBRATION_H
#define XML_ALGORITHM_CALIBRATION_H

#include <string>
#include <vector>
#include <iostream>
#include <list>

/**
 *  This is the RAVE version of the AlgorithmCalibration code.
*/

template <class T,class CO> class AlgorithmCalibration
{
  public:

  /**
   * Create an AlgorithmCalibration class and load from fileName the
   * cateogries and their objects.
   */
    AlgorithmCalibration() {};
    AlgorithmCalibration ( const std::vector<std::pair<T*,CO*> > & );
    
    ~AlgorithmCalibration();
    
   /**
    * Get the calibratedObject that belong to the category that matches the given input
    */ 
    const CO* getCalibData ( const typename T::Input & calibrationInput ) const;

    std::string filename() const { return "AlgorithmCalibration"; };

    const std::vector<std::pair<T*,CO*> > & dumpCalibration() const;

 protected:

    std::pair<T *,CO*> searchCategory(const typename T::Input & calibrationInput) const ;

 protected:

    void addCategory(T * newCategory,CO * newCalibratedObject)
    {
       m_categoriesWithObjects.push_back(std::pair<T*,CO*>(newCategory,newCalibratedObject));
    }


  private:
    std::vector<std::pair<T*,CO*> > m_categoriesWithObjects;
};


template < class T, class CO >
AlgorithmCalibration<T,CO>::AlgorithmCalibration( const std::vector<std::pair<T*,CO*> > & c ) :
  m_categoriesWithObjects ( c )
{}

template <class T,class CO> 
AlgorithmCalibration<T,CO>::~AlgorithmCalibration()
{
  // FIXME needs to delete, but not now
  /*
   for(typename std::vector<std::pair<T*,CO*> >::iterator it = m_categoriesWithObjects.begin();it!=m_categoriesWithObjects.end();it++)
   {
    delete (*it).first;
    delete (*it).second;
   }
   */
}
    
template <class T,class CO> 
std::pair<T *,CO*>  AlgorithmCalibration<T,CO>::searchCategory(const typename T::Input & calibrationInput) const
{
   std::pair<T *,CO*> categoryWithObject(0,0);
   for(typename std::vector<std::pair<T *,CO *> >::const_iterator it = m_categoriesWithObjects.begin();it!=m_categoriesWithObjects.end();it++)
   {
      if((*it).first->match(calibrationInput))
      {
        if(categoryWithObject.first!=0) cout << "WARNING: OVERLAP in categories, using latest one" << endl;
        categoryWithObject=*it;
      }
   }
  return categoryWithObject;
}

template <class T,class CO> 
const CO * AlgorithmCalibration<T,CO>::getCalibData(const typename T::Input & calibrationInput) const
{
  return searchCategory(calibrationInput).second; 
}

template <class T,class CO> 
const std::vector<std::pair<T*,CO*> > & AlgorithmCalibration<T,CO>::dumpCalibration() const
{      
  return m_categoriesWithObjects;
}


#endif
