#ifndef CalibratedHistogramXML_H
#define CalibratedHistogramXML_H
#include "RecoBTag/XMLCalibration/interface/CalibratedObject.h"
#include <vector>

/**
* This class implements some methods of the CalibratedObject.
* This class does not provide methdos for calibration, i.e.
*  [start|update|finish]Calibration() functions.
* If you want to use it in a calibration program you have to 
* implement those methods in a child class.
*/
using namespace std;
class CalibratedHistogramXML:public CalibratedObject
{
public:
  CalibratedHistogramXML() {} 
  virtual ~CalibratedHistogramXML() {} 
  CalibratedHistogramXML(vector<float> binLimits) : m_binULimits(binLimits),m_normalization(0) 
  { }

  std::string name () const
  {
    return "CalibratedHistogramXML";
  }

   /*
    zero is underflow, size is overflow (the number of good bins is size-1))
   */
  float binContent(int bin) const {return m_binValues[bin];}
  float value(float x) const {return binContent(findBin(x));}
  void  reset();
  void  setBinContent(int b,float v);
  void  fill(float x, float w=1.) 
  { 
   int bin=findBin(x);
   setBinContent(bin,binContent(bin)+w);
  }
  
  int findBin(float x) const ;
  void read() {};
  void write() const {};

  float integral(float hBound, float lBound=0,int mode=0) const ;
  float normalizedIntegral(float hBound, float lBound=0,int mode=0) const
  {
   return integral(hBound,lBound,mode)/m_normalization;
  } 

  std::vector<float> upperLimits() const;

  void dump() const;
  
protected:
  vector<float> m_binULimits;
  vector<float> m_binValues;
  float m_normalization;
};

#endif
