#ifndef _TransientTrack_H_
#define _TransientTrack_H_

#include "TrackingTools/TrajectoryState/interface/TrajectoryStateOnSurface.h"
#include "TrackingTools/TrajectoryState/interface/FreeTrajectoryState.h"
#include "TrackingTools/TrajectoryState/interface/TrajectoryStateClosestToPoint.h"
#include "TrackingTools/TrajectoryParametrization/interface/GlobalTrajectoryParameters.h"
#include "DataFormats/TrackReco/interface/HelixParameters.h"
#include <rave/RaveTrack.h>
#include "MagneticField/Engine/interface/MagneticField.h"
#include "DataFormats/TrackReco/interface/Track.h"

namespace reco
{

/*!
 * Definition of Transient Track class for
 * reconstruction posterior to track reconstruction (vertexing, b-tagging...)
 */
class TransientTrack : public Track
{
public:
    //! constructor from GlobalTrajectoryParameters
    TransientTrack( const FreeTrajectoryState & f, const GlobalPoint & gp );
    TransientTrack( const TransientTrack & tt );
    TransientTrack( const rave::Track & );
    TransientTrack();

    // TransientTrack& operator= ( const TransientTrack & tt );

    /*
    TrajectoryStateOnSurface outermostMeasurementState();

    TrajectoryStateOnSurface innermostMeasurementState();

    
    TrajectoryStateClosestToPoint
    trajectoryStateClosestToPoint( const GlobalPoint & point ) const
    {
        return builder(originalTSCP.theState(), point);
    }

    TrajectoryStateClosestToPoint impactPointTSCP() const
    {
        return originalTSCP;
    }
    
    */
    const MagneticField * field() const;
    
    const TrackCharge charge() const;

    bool operator== ( const TransientTrack & other ) const;
    
    bool operator< ( const TransientTrack & other ) const;
    
    TrajectoryStateOnSurface impactPointState() const;
    
    FreeTrajectoryState initialFreeState() const { return fts; };

    TrajectoryStateOnSurface innermostMeasurementState() const;

    const TransientTrack & track() const;

    double pt() const;

    int id() const;

    double chi2() const;
    double ndof() const;

    rave::Track raveTrack() const;

private:    
    void calculateStateAtVertex() const;

private:
    FreeTrajectoryState fts;    //!< Holds the main track data
    GlobalPoint refPoint;       //!< Holds the reference point for the track

    // TrajectoryStateClosestToPoint originalTSCP;
    
    mutable bool stateAtVertexAvailable;
    
    mutable TrajectoryStateOnSurface theStateAtVertex;
    int theId;
    double theChi2;
    double theNdof;
    rave::Track theRTrack;
};

}

#endif
